@@ -1,7 +1,7 @@
 #
 # Run this Tcl script to generate the lang-*.html files.
 #
-set rcsid {$Id: lang.tcl,v 1.108 2006/02/09 22:13:42 drh Exp $}
+set rcsid {$Id: lang.tcl,v 1.133 2007/08/09 00:00:26 drh Exp $}
 source common.tcl
 
 if {[llength $argv]>0} {
@@ -12,7 +12,7 @@ if {[llength $argv]>0} {
 
 header {Query Language Understood by SQLite}
 puts {
-<h1>SQL As Understood By SQLite</h1>
+<h1 class="pdf_section">SQL As Understood By SQLite</h1>
 
 <p>The SQLite library understands most of the standard SQL
 language.  But it does <a href="omitted.html">omit some features</a>
@@ -31,7 +31,7 @@ by SQLite.  Many low-level productions are omitted.  For detailed information
 on the language that SQLite understands, refer to the source code and
 the grammar file "parse.y".</p>
 
-
+<div class="pdf_ignore">
 <p>SQLite implements the follow syntax:</p>
 <p><ul>
 }
@@ -49,6 +49,7 @@ proc slink {label} {
 
 foreach {section} [lsort -index 0 -dictionary {
   {{CREATE TABLE} createtable}
+  {{CREATE VIRTUAL TABLE} createvtab}
   {{CREATE INDEX} createindex}
   {VACUUM vacuum}
   {{DROP TABLE} droptable}
@@ -82,6 +83,7 @@ foreach {section} [lsort -index 0 -dictionary {
   puts "<li><a href=\"[slink $s_tag]\">$s_title</a></li>"
 }
 puts {</ul></p>
+</div>
 
 <p>Details on the implementation of each command are provided in
 the sequel.</p>
@@ -214,7 +216,7 @@ table named <b>sqlite_stat1</b>.  Future enhancements may create
 additional tables with the same name pattern except with the "1"
 changed to a different digit.  The <b>sqlite_stat1</b> table cannot
 be <a href="#droptable">DROP</a>ped,
-but it all the content can be <a href="#delete">DELETE</a>d which has the
+but all the content can be <a href="#delete">DELETE</a>d which has the
 same effect.</p>
 }
 
@@ -225,7 +227,7 @@ ATTACH [DATABASE] <database-filename> AS <database-name>
 }
 
 puts {
-<p>The ATTACH DATABASE statement adds a preexisting database 
+<p>The ATTACH DATABASE statement adds another database 
 file to the current database connection.  If the filename contains 
 punctuation characters it must be quoted.  The names 'main' and 
 'temp' refer to the main database and the database used for 
@@ -247,9 +249,9 @@ also permissible to attach the same database file multiple times.</p>
 <i>database-name.table-name</i>.  If an attached table doesn't have 
 a duplicate table name in the main database, it doesn't require a 
 database name prefix.  When a database is attached, all of its 
-tables which don't have duplicate names become the 'default' table
+tables which don't have duplicate names become the default table
 of that name.  Any tables of that name attached afterwards require the table 
-prefix. If the 'default' table of a given name is detached, then 
+prefix. If the default table of a given name is detached, then 
 the last table of that name attached becomes the new default.</p>
 
 <p>
@@ -487,7 +489,7 @@ command.</p>
 Section {CREATE TABLE} {createtable}
 
 Syntax {sql-command} {
-CREATE [TEMP | TEMPORARY] TABLE [IF NOT EXISTS] <table-name> (
+CREATE [TEMP | TEMPORARY] TABLE [IF NOT EXISTS] [<database-name> .] <table-name> (
   <column-def> [, <column-def>]*
   [, <constraint>]*
 )
@@ -553,16 +555,29 @@ may only hold unique integer values.  (Except for this one case,
 SQLite ignores the datatype specification of columns and allows
 any kind of data to be put in a column regardless of its declared
 datatype.)  If a table does not have an INTEGER PRIMARY KEY column,
-then the B-Tree key will be a automatically generated integer.  The
+then the B-Tree key will be a automatically generated integer.
+<a name="rowid"> The
 B-Tree key for a row can always be accessed using one of the
 special names "<b>ROWID</b>", "<b>OID</b>", or "<b>_ROWID_</b>".
 This is true regardless of whether or not there is an INTEGER
-PRIMARY KEY.  An INTEGER PRIMARY KEY column man also include the
+PRIMARY KEY.  An INTEGER PRIMARY KEY column can also include the
 keyword AUTOINCREMENT.  The AUTOINCREMENT keyword modified the way
 that B-Tree keys are automatically generated.  Additional detail
 on automatic B-Tree key generation is available
 <a href="autoinc.html">separately</a>.</p>
 
+<p>According to the SQL standard, PRIMARY KEY should imply NOT NULL.
+Unfortunately, due to a long-standing coding oversight, this is not 
+the case in SQLite.  SQLite allows NULL values
+in a PRIMARY KEY column.  We could change SQLite to conform to the
+standard (and we might do so in the future), but by the time the
+oversight was discovered, SQLite was in such wide use that we feared
+breaking legacy code if we fixed the problem.  So for now we have
+chosen to contain allowing NULLs in PRIMARY KEY columns.
+Developers should be aware, however, that we may change SQLite to
+conform to the SQL standard in future and should design new programs
+accordingly.</p>
+
 <p>If the "TEMP" or "TEMPORARY" keyword occurs in between "CREATE"
 and "TABLE" then the table that is created is only visible
 within that same database connection
@@ -625,13 +640,13 @@ statement.  </p>
 Section {CREATE TRIGGER} createtrigger
 
 Syntax {sql-statement} {
-CREATE [TEMP | TEMPORARY] TRIGGER <trigger-name> [ BEFORE | AFTER ]
+CREATE [TEMP | TEMPORARY] TRIGGER [IF NOT EXISTS] <trigger-name> [ BEFORE | AFTER ]
 <database-event> ON [<database-name> .] <table-name>
 <trigger-action>
 }
 
 Syntax {sql-statement} {
-CREATE [TEMP | TEMPORARY] TRIGGER <trigger-name> INSTEAD OF
+CREATE [TEMP | TEMPORARY] TRIGGER [IF NOT EXISTS] <trigger-name> INSTEAD OF
 <database-event> ON [<database-name> .] <view-name>
 <trigger-action>
 }
@@ -644,7 +659,7 @@ UPDATE OF <column-list>
 }
 
 Syntax {trigger-action} {
-[ FOR EACH ROW | FOR EACH STATEMENT ] [ WHEN <expression> ] 
+[ FOR EACH ROW ] [ WHEN <expression> ] 
 BEGIN 
   <trigger-step> ; [ <trigger-step> ; ]*
 END
@@ -781,7 +796,7 @@ statement.</p>
 Section {CREATE VIEW} {createview}
 
 Syntax {sql-command} {
-CREATE [TEMP | TEMPORARY] VIEW [<database-name>.] <view-name> AS <select-statement>
+CREATE [TEMP | TEMPORARY] VIEW [IF NOT EXISTS] [<database-name>.] <view-name> AS <select-statement>
 }
 
 puts {
@@ -809,6 +824,40 @@ with the <a href="#dropview">DROP VIEW</a>
 command.</p>
 }
 
+Section {CREATE VIRTUAL TABLE} {createvtab}
+
+Syntax {sql-command} {
+CREATE VIRTUAL TABLE [<database-name> .] <table-name> USING <module-name> [( <arguments> )]
+}
+
+puts {
+<p>A virtual table is an interface to an external storage or computation
+engine that appears to be a table but does not actually store information
+in the database file.</p>
+
+<p>In general, you can do anything with a virtual table that can be done
+with an ordinary table, except that you cannot create triggers on a
+virtual table.  Some virtual table implementations might impose additional
+restrictions.  For example, many virtual tables are read-only.</p>
+
+<p>The &lt;module-name&gt; is the name of an object that implements
+the virtual table.  The &lt;module-name&gt; must be registered with
+the SQLite database connection using
+<a href="capi3ref.html#sqlite3_create_module">sqlite3_create_module</a>
+prior to issuing the CREATE VIRTUAL TABLE statement.
+The module takes zero or more comma-separated arguments.
+The arguments can be just about any text as long as it has balanced
+parentheses.  The argument syntax is sufficiently general that the
+arguments can be made to appear as column definitions in a traditional
+<a href="#createtable">CREATE TABLE</a> statement.  
+SQLite passes the module arguments directly
+to the module without any interpretation.  It is the responsibility
+of the module implementation to parse and interpret its own arguments.</p>
+
+<p>A virtual table is destroyed using the ordinary
+<a href="#droptable">DROP TABLE</a> statement.  There is no
+DROP VIRTUAL TABLE statement.</p>
+}
 
 Section DELETE delete
 
@@ -894,7 +943,7 @@ result if the table does not exist.</p>
 
 Section {DROP TRIGGER} droptrigger
 Syntax {sql-statement} {
-DROP TRIGGER [<database-name> .] <trigger-name>
+DROP TRIGGER [IF EXISTS] [<database-name> .] <trigger-name>
 }
 puts { 
 <p>The DROP TRIGGER statement removes a trigger created by the 
@@ -907,7 +956,7 @@ dropped when the associated table is dropped.</p>
 Section {DROP VIEW} dropview
 
 Syntax {sql-command} {
-DROP VIEW <view-name>
+DROP VIEW [IF EXISTS] <view-name>
 }
 
 puts {
@@ -960,9 +1009,10 @@ Syntax {expr} {
 <expr> [NOT] IN [<database-name> .] <table-name> |
 [EXISTS] ( <select-statement> ) |
 CASE [<expr>] LP WHEN <expr> THEN <expr> RPPLUS [ELSE <expr>] END |
-CAST ( <expr> AS <type> )
+CAST ( <expr> AS <type> ) |
+<expr> COLLATE <collation-name>
 } {like-op} {
-LIKE | GLOB | REGEXP
+LIKE | GLOB | REGEXP | MATCH
 }
 
 puts {
@@ -985,12 +1035,21 @@ AND
 OR</font>
 </pre></blockquote>
 
-<p>Supported unary operators are these:</p>
+<p>Supported unary prefix operators are these:</p>
 
 <blockquote><pre>
-<font color="#2c2cf0"><big>-    +    !    ~</big></font>
+<font color="#2c2cf0"><big>-    +    !    ~    NOT</big></font>
 </pre></blockquote>
 
+<p>The COLLATE operator can be thought of as a unary postfix
+operator.  The COLLATE operator has the highest precedence.
+It always binds more tightly than any prefix unary operator or
+any binary operator.</p>
+
+<p>The unary operator [Operator +] is a no-op.  It can be applied
+to strings, numbers, or blobs and it always gives as its result the
+value of the operand.</p>
+
 <p>Note that there are two variations of the equals and not equals
 operators.  Equals can be either}
 puts "[Operator =] or [Operator ==].
@@ -1022,7 +1081,7 @@ BLOB literals are string literals containing hexadecimal data and
 preceded by a single "x" or "X" character.  For example:</p>
 
 <blockquote><pre>
-X'53514697465'
+X'53514C697465'
 </pre></blockquote>
 
 <p>
@@ -1037,7 +1096,7 @@ Parameters can take several forms:
 </p
 
 <blockquote>
-<table>
+<table class="pdf_functions">
 <tr>
 <td align="right" valign="top"><b>?</b><i>NNN</i></td><td width="20"></td>
 <td>A question mark followed by a number <i>NNN</i> holds a spot for the
@@ -1056,6 +1115,10 @@ The number assigned is the next unused number.  To avoid confusion,
 it is best to avoid mixing named and numbered parameters.</td>
 </tr>
 <tr>
+<td align="right" valign="top"><b>@</b><i>AAAA</i></td><td width="20"></td>
+<td>An "at" sign works exactly like a colon.</td>
+</tr>
+<tr>
 <td align="right" valign="top"><b>$</b><i>AAAA</i></td><td width="20"></td>
 <td>A dollar-sign followed by an identifier name also holds a spot for a named
 parameter with the name AAAA.  The identifier name in this case can include
@@ -1127,14 +1190,21 @@ that function.</p>
 user function.  No regexp() user function is defined by default
 and so use of the REGEXP operator will normally result in an
 error message.  If a user-defined function named "regexp"
-is defined at run-time, that function will be called in order
+is added at run-time, that function will be called in order
 to implement the REGEXP operator.</p>
 
+<a name="match"></a>
+<p>The MATCH operator is a special syntax for the match()
+user function.  The default match() function implementation
+raises and exception and is not really useful for anything.
+But extensions can override the match() function with more
+helpful logic.</p>
+
 <p>A column name can be any of the names defined in the CREATE TABLE
 statement or one of the following special identifiers: "<b>ROWID</b>",
 "<b>OID</b>", or "<b>_ROWID_</b>".
 These special identifiers all describe the
-unique random integer key (the "row key") associated with every 
+unique integer key (the "row key") associated with every 
 row of every table.
 The special identifiers only refer to the row key if the CREATE TABLE
 statement does not define a real column with the same name.  Row keys
@@ -1181,12 +1251,15 @@ a result immediately based on their inputs.  Aggregate functions
 may only be used in a SELECT statement.  Aggregate functions compute
 their result across all rows of the result set.</p>
 
-<p>The functions shown below are available by default.  Additional
+<a name="corefunctions"></a>
+<b>Core Functions</b>
+
+<p>The core functions shown below are available by default.  Additional
 functions may be written in C and added to the database engine using
 the <a href="capi3ref.html#cfunc">sqlite3_create_function()</a>
 API.</p>
 
-<table border=0 cellpadding=10>
+<table border=0 cellpadding=10 class="pdf_functions">
 <tr>
 <td valign="top" align="right" width=120>abs(<i>X</i>)</td>
 <td valign="top">Return the absolute value of argument <i>X</i>.</td>
@@ -1200,8 +1273,9 @@ all arguments are NULL then NULL is returned.  There must be at least
 </tr>
 
 <tr>
+<td valign="top" align="right">
 <a name="globFunc"></a>
-<td valign="top" align="right">glob(<i>X</i>,<i>Y</i>)</td>
+glob(<i>X</i>,<i>Y</i>)</td>
 <td valign="top">This function is used to implement the
 "<b>X GLOB Y</b>" syntax of SQLite.  The
 <a href="capi3ref.html#sqlite3_create_function">sqlite3_create_function()</a> 
@@ -1217,9 +1291,18 @@ both arguments are NULL then NULL is returned. This behaves the same as
 <b>coalesce()</b> above.</td>
 </tr>
 
+<tr>
+<td valign="top" align="right">
+<a name="hexFunc">
+hex(<i>X</i>)</td>
+<td valign="top">The argument is interpreted as a BLOB.  The result
+is a hexadecimal rendering of the content of that blob.</td>
+</tr>
+
 <tr>
 <td valign="top" align="right">last_insert_rowid()</td>
-<td valign="top">Return the ROWID of the last row insert from this
+<td valign="top">Return the <a href="lang_createtable.html#rowid">ROWID</a>
+of the last row insert from this
 connection to the database.  This is the same value that would be returned
 from the <b>sqlite_last_insert_rowid()</b> API function.</td>
 </tr>
@@ -1232,8 +1315,10 @@ characters is returned, not the number of bytes.</td>
 </tr>
 
 <tr>
+<td valign="top" align="right">
 <a name="likeFunc"></a>
-<td valign="top" align="right">like(<i>X</i>,<i>Y</i> [,<i>Z</i>])</td>
+like(<i>X</i>,<i>Y</i>)<br>
+like(<i>X</i>,<i>Y</i>,<i>Z</i>)</td>
 <td valign="top">
 This function is used to implement the "<b>X LIKE Y [ESCAPE Z]</b>"
 syntax of SQL. If the optional ESCAPE clause is present, then the
@@ -1249,6 +1334,16 @@ LIKE operator depending on whether or not an ESCAPE clause was
 specified.</td>
 </tr>
 
+<tr>
+<td valign="top" align="right">load_extension(<i>X</i>)<br>
+load_extension(<i>X</i>,<i>Y</i>)</td>
+<td valign="top">Load SQLite extensions out of the shared library
+file named <i>X</i> using the entry point <i>Y</i>.  The result
+is a NULL.  If <i>Y</i> is omitted then the default entry point
+of <b>sqlite3_extension_init</b> is used.  This function raises
+an exception if the extension fails to load or initialize correctly.
+</tr>
+
 <tr>
 <td valign="top" align="right">lower(<i>X</i>)</td>
 <td valign="top">Return a copy of string <i>X</i> will all characters
@@ -1257,6 +1352,16 @@ for the conversion, which means that this function might not
 work correctly on UTF-8 characters.</td>
 </tr>
 
+<tr>
+<td valign="top" align="right">
+<a name="ltrimFunc">
+ltrim(<i>X</i>)<br>ltrim(<i>X</i>,<i>Y</i>)</td>
+<td valign="top">Return a string formed by removing any and all
+characters that appear in <i>Y</i> from the left side of <i>X</i>.
+If the <i>Y</i> argument is omitted, spaces are removed.</td>
+</tr>
+
+
 <tr>
 <td valign="top" align="right">max(<i>X</i>,<i>Y</i>,...)</td>
 <td valign="top">Return the argument with the maximum value.  Arguments
@@ -1294,8 +1399,25 @@ is also useful when writing triggers to implement undo/redo functionality.
 
 <tr>
 <td valign="top" align="right">random(*)</td>
-<td valign="top">Return a random integer between -2147483648 and
-+2147483647.</td>
+<td valign="top">Return a pseudo-random integer
+between -9223372036854775808 and +9223372036854775807.</td>
+</tr>
+
+<tr>
+<td valign="top" align="right">
+<a name="replaceFunc">
+replace(<i>X</i>,<i>Y</i>,<i>Z</i>)</td>
+<td valign="top">Return a string formed by substituting string <i>Z</i> for
+every occurrance of string <i>Y</i> in string <i>X</i>.  The BINARY
+collating sequence is used for comparisons.</td>
+</tr>
+
+<tr>
+<td valign="top" align="right">
+<a name="randomblobFunc">
+randomblob(<i>N</i>)</td>
+<td valign="top">Return a <i>N</i>-byte blob containing pseudo-random bytes.
+<i>N</i> should be a postive integer.</td>
 </tr>
 
 <tr>
@@ -1305,6 +1427,15 @@ right of the decimal point.  If the <i>Y</i> argument is omitted, 0 is
 assumed.</td>
 </tr>
 
+<tr>
+<td valign="top" align="right">
+<a name="rtrimFunc">
+rtrim(<i>X</i>)<br>rtrim(<i>X</i>,<i>Y</i>)</td>
+<td valign="top">Return a string formed by removing any and all
+characters that appear in <i>Y</i> from the right side of <i>X</i>.
+If the <i>Y</i> argument is omitted, spaces are removed.</td>
+</tr>
+
 <tr>
 <td valign="top" align="right">soundex(<i>X</i>)</td>
 <td valign="top">Compute the soundex encoding of the string <i>X</i>.
@@ -1326,10 +1457,21 @@ that is running.  Example:  "2.8.0"</td>
 with the <i>Y</i>-th character and which is <i>Z</i> characters long.
 The left-most character of <i>X</i> is number 1.  If <i>Y</i> is negative
 the the first character of the substring is found by counting from the
-right rather than the left.  If SQLite is configured to support UTF-8,
-then characters indices refer to actual UTF-8 characters, not bytes.</td>
+right rather than the left.  If <i>X</i> is string
+then characters indices refer to actual UTF-8 characters.  If
+<i>X</i> is a BLOB then the indices refer to bytes.</td>
 </tr>
 
+<tr>
+<td valign="top" align="right">
+<a name="trimFunc">
+trim(<i>X</i>)<br>trim(<i>X</i>,<i>Y</i>)</td>
+<td valign="top">Return a string formed by removing any and all
+characters that appear in <i>Y</i> from both ends of <i>X</i>.
+If the <i>Y</i> argument is omitted, spaces are removed.</td>
+</tr>
+
+
 <tr>
 <td valign="top" align="right">typeof(<i>X</i>)</td>
 <td valign="top">Return the type of the expression <i>X</i>.  The only 
@@ -1345,8 +1487,27 @@ upper-case letters.  The implementation of this function uses the C library
 routine <b>toupper()</b> which means it may not work correctly on 
 UTF-8 strings.</td>
 </tr>
+
+<tr>
+<td valign="top" align="right">zeroblob(<i>N</i>)</td>
+<td valign="top"><a name="zeroblob">
+Return a BLOB consisting of N bytes of 0x00.  SQLite
+manages these zeroblobs very efficiently.  Zeroblobs can be used to
+reserve space for a BLOB that is later written using 
+<a href="capi3ref.html#sqlite3_blob_open">incremental BLOB I/O</a>.</td>
+</tr>
+
 </table>
 
+<b>Date And Time Functions</b>
+
+<p>Date and time functions are documented in the 
+<a href="http://www.sqlite.org/cvstrac/wiki?p=DateAndTimeFunctions">
+SQLite Wiki</a>.</p>
+
+<a name="aggregatefunctions"></a>
+<b>Aggregate Functions</b>
+
 <p>
 The aggregate functions shown below are available by default.  Additional
 aggregate functions written in C may be added using the 
@@ -1362,7 +1523,7 @@ of distinct values of column X instead of the total number of non-null
 values in column X.
 </p>
 
-<table border=0 cellpadding=10>
+<table border=0 cellpadding=10 class="pdf_functions">
 <tr>
 <td valign="top" align="right" width=120>avg(<i>X</i>)</td>
 <td valign="top">Return the average value of all non-NULL <i>X</i> within a
@@ -1405,12 +1566,15 @@ if all values in the group are NULL.</td>
    in the SQL language.</p>
 
    <p>The result of total() is always a floating point value.
-   The result of sum() is an integer value if all non-NULL inputs are integers
-   and the sum is exact.  If any input to sum() is neither an integer or
-   a NULL or if the
-   an integer overflow occurs at any point during the computation,
+   The result of sum() is an integer value if all non-NULL inputs are integers.
+   If any input to sum() is neither an integer or a NULL
    then sum() returns a floating point value
-   which might be an approximation to the true sum.</td>
+   which might be an approximation to the true sum.</p>
+
+   <p>Sum() will throw an "integer overflow" exception if all inputs
+   are integers or NULL
+   and an integer overflow occurs at any point during the computation.
+   Total() never throws an exception.</p>
 </tr>
 </table>
 }
@@ -1430,7 +1594,7 @@ If no column-list is specified then the number of values must
 be the same as the number of columns in the table.  If a column-list
 is specified, then the number of values must match the number of
 specified columns.  Columns of the table that do not appear in the
-column list are filled with the default value, or with NULL if not
+column list are filled with the default value, or with NULL if no
 default value is specified.
 </p>
 
@@ -1440,8 +1604,7 @@ SELECT must exactly match the number of columns in the table if
 no column list is specified, or it must match the number of columns
 name in the column list.  A new entry is made in the table
 for every row of the SELECT result.  The SELECT may be simple
-or compound.  If the SELECT statement has an ORDER BY clause,
-the ORDER BY is ignored.</p>
+or compound.</p>
 
 <p>The optional conflict-clause allows the specification of an alternative
 constraint conflict resolution algorithm to use during this one command.
@@ -1518,11 +1681,12 @@ or updating the current row.  Thus the insert or update always occurs.
 The command continues executing normally.  No error is returned.
 If a NOT NULL constraint violation occurs, the NULL value is replaced
 by the default value for that column.  If the column has no default
-value, then the ABORT algorithm is used.</p>
+value, then the ABORT algorithm is used.  If a CHECK constraint violation
+occurs then the IGNORE algorithm is used.</p>
 
 <p>When this conflict resolution strategy deletes rows in order to
 satisfy a constraint, it does not invoke delete triggers on those
-rows.  But that may change in a future release.</p>
+rows.  This behavior might change in a future release.</p>
 </dl>
 
 <p>The algorithm specified in the OR clause of a INSERT or UPDATE
@@ -1738,15 +1902,27 @@ The VACUUM command cleans
 the main database by copying its contents to a temporary database file and 
 reloading the original database file from the copy.  This eliminates 
 free pages,  aligns table data to be contiguous, and otherwise cleans 
-up the database file structure. It is not possible to perform the same
-process on an attached database file.</p>
+up the database file structure.</p>
 
-<p>This command will fail if there is an active transaction.  This 
-command has no effect on an in-memory database.</p>
+<p>The VACUUM command may change the 
+<a href="lang_createtable.html#rowid">ROWID</a> of entires in tables that do
+not have an explicit INTEGER PRIMARY KEY.</p>
+
+<p>VACUUM only works on the main database.
+It is not possible to VACUUM an attached database file.</p>
+
+<p>The VACUUM command will fail if there is an active transaction.
+The VACUUM command is a no-op for in-memory databases.</p>
 
 <p>As of SQLite version 3.1, an alternative to using the VACUUM command
 is auto-vacuum mode, enabled using the 
-<a href="pragma.html#pragma_auto_vacuum">auto_vacuum pragma</a>.</p>
+<a href="pragma.html#pragma_auto_vacuum">auto_vacuum pragma</a>.
+When auto-vacuum is enabled for a database, large deletes cause
+the size of the database file to shrink.  However, auto-vacuum
+also causes excess fragmentation of the database file.  And auto-vacuum
+does not compact partially filled pages of the database as VACUUM
+does.
+</p>
 }
 
 # A list of keywords.  A asterisk occurs after the keyword if it is on
@@ -1754,9 +1930,11 @@ is auto-vacuum mode, enabled using the
 #
 set keyword_list [lsort {
    ABORT*
+   ADD
    AFTER*
    ALL
    ALTER
+   ANALYZE*
    AND
    AS
    ASC*
@@ -1768,6 +1946,7 @@ set keyword_list [lsort {
    BY
    CASCADE*
    CASE
+   CAST*
    CHECK
    COLLATE
    COMMIT
@@ -1802,6 +1981,7 @@ set keyword_list [lsort {
    GLOB*
    GROUP
    HAVING
+   IF*
    IGNORE*
    IMMEDIATE*
    IN
@@ -1830,8 +2010,10 @@ set keyword_list [lsort {
    OR
    ORDER
    OUTER
+   PLAN*
    PRAGMA*
    PRIMARY
+   QUERY*
    RAISE*
    REFERENCES
    REINDEX*
@@ -1843,7 +2025,6 @@ set keyword_list [lsort {
    ROW*
    SELECT
    SET
-   STATEMENT*
    TABLE
    TEMP*
    TEMPORARY*
@@ -1858,27 +2039,31 @@ set keyword_list [lsort {
    VACUUM*
    VALUES
    VIEW*
+   VIRTUAL*
    WHEN
    WHERE
 }]
 
 
 
-Section {SQLite keywords} keywords
+puts {<DIV class="pdf_section">}
+Section {SQLite Keywords} keywords 
+puts {</DIV>}
 
 puts {
 <p>The SQL standard specifies a huge number of keywords which may not
-be used as the names of tables, indices, columns, or databases.  The
-list is so long that few people can remember them all.  For most SQL
-code, your safest bet is to never use any English language word as the
-name of a user-defined object.</p>
+be used as the names of tables, indices, columns, databases, user-defined
+functions, collations, virtual table modules, or any other named object.
+The list of keywords is so long that few people can remember them all.
+For most SQL code, your safest bet is to never use any English language
+word as the name of a user-defined object.</p>
 
 <p>If you want to use a keyword as a name, you need to quote it.  There
 are three ways of quoting keywords in SQLite:</p>
 
 <p>
 <blockquote>
-<table>
+<table class="pdf_functions">
 <tr>	<td valign="top"><b>'keyword'</b></td><td width="20"></td>
 	<td>A keyword in single quotes is interpreted as a literal string
         if it occurs in a context where a string literal is allowed, otherwise
@@ -1898,14 +2083,16 @@ are three ways of quoting keywords in SQLite:</p>
 
 <p>Quoted keywords are unaesthetic.
 To help you avoid them, SQLite allows many keywords to be used unquoted
-as the names of databases, tables, indices, triggers, views, and/or columns.
+as the names of databases, tables, indices, triggers, views, columns,
+user-defined functions, collations, attached databases, and virtual
+function modules.
 In the list of keywords that follows, those that can be used as identifiers
 are shown in an italic font.  Keywords that must be quoted in order to be
 used as identifiers are shown in bold.</p>
 
 <p>
 SQLite adds new keywords from time to time when it take on new features.
-So to prevent you code from being broken by future enhancements, you should
+So to prevent your code from being broken by future enhancements, you should
 normally quote any indentifier that is an English language word, even if
 you do not have to.
 </p>
@@ -1915,7 +2102,7 @@ The following are the keywords currently recognized by SQLite:
 </p>
 
 <blockquote>
-<table width="100%">
+<table width="100%" class="pdf_keywords">
 <tr>
 <td align="left" valign="top" width="20%">
 }
@@ -1949,7 +2136,7 @@ puts {
 system objects.  They can be used as an identifier for a different 
 type of object.</p>
 
-<blockquote><b>
+<blockquote class="pdf_keywords"><b>
   _ROWID_<br>
   MAIN<br>
   OID<br>
@@ -1961,7 +2148,9 @@ type of object.</p>
 </b></blockquote>
 }
 
+puts {<DIV class="pdf_ignore">}
 footer $rcsid
 if {[string length $outputdir]} {
   footer $rcsid
 }
+puts {</DIV>}
